package com.example.playertime;

import com.google.gson.*;
import com.mojang.authlib.GameProfile;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.network.ServerPlayerEntity;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class PlayerTimeTracker {
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();
    private final MinecraftServer server;
    private final Path dataFile;
    private final ExecutorService executor = Executors.newSingleThreadExecutor();
    private final Map<UUID, PlayerTimeData> playerData = new ConcurrentHashMap<>();

    public PlayerTimeTracker(MinecraftServer server) {
        this.server = server;
        this.dataFile = server.getRunDirectory().resolve("player_time_data.json");
        loadData();
    }

    private boolean isWhitelisted(String playerName) {
        MinecraftServer server = this.server;
        if (server == null) return false;

        return server.getPlayerManager().getWhitelist()
                .isAllowed(server.getUserCache().findByName(playerName).orElse(null));
    }


    public void onPlayerJoin(ServerPlayerEntity player) {
        PlayerTimeData data = playerData.computeIfAbsent(player.getUuid(), uuid -> new PlayerTimeData());
        data.lastLogin = Instant.now().getEpochSecond();
        saveAsync(player.getUuid());
    }

    public void onPlayerLeave(ServerPlayerEntity player) {
        PlayerTimeData data = playerData.get(player.getUuid());
        if (data != null) {
            long now = Instant.now().getEpochSecond();
            long sessionTime = now - data.lastLogin;
            data.totalTime += sessionTime;

            // 维护30天滚动窗口
            data.rolling30Days.addPlayTime(now, sessionTime);
            data.rolling7Days.addPlayTime(now, sessionTime);

            data.lastLogin = 0;
            saveAsync(player.getUuid());
        }
    }

    public PlayerTimeStats getPlayerStats(UUID uuid) {
        PlayerTimeData data = playerData.get(uuid);
        if (data == null) {
            return null;
        }

        long now = Instant.now().getEpochSecond();
        PlayerTimeStats stats = new PlayerTimeStats();
        stats.totalTime = data.totalTime;

        // 如果玩家在线，添加当前会话时间
        if (data.lastLogin > 0) {
            stats.totalTime += (now - data.lastLogin);
        }

        stats.last30Days = data.rolling30Days.getTotalTime(now);
        stats.last7Days = data.rolling7Days.getTotalTime(now);

        return stats;
    }

    public Map<String, String> getWhitelistedPlayerStats() {
        Map<String, String> stats = new LinkedHashMap<>();
        long now = Instant.now().getEpochSecond();

        // 获取白名单玩家UUID集合
        Set<UUID> whitelistUuids = new HashSet<>();
        for (String name : server.getPlayerManager().getWhitelist().getNames()) {
            server.getUserCache().findByName(name).ifPresent(profile -> {
                whitelistUuids.add(profile.getId());
            });
        }

        // 遍历所有已记录玩家
        playerData.forEach((uuid, data) -> {
            if (whitelistUuids.contains(uuid)) {
                String playerName = getPlayerName(uuid);
                long totalTime = data.totalTime;
                if (data.lastLogin > 0) {
                    totalTime += (now - data.lastLogin);
                }

                stats.put(playerName, "总时长: " + formatTime(totalTime) +
                        " | 30天: " + formatTime(data.rolling30Days.getTotalTime(now)) +
                        " | 7天: " + formatTime(data.rolling7Days.getTotalTime(now)));
            }
        });

        return stats;
    }


    public String getPlayerName(UUID uuid) {
        // 尝试获取在线玩家
        ServerPlayerEntity player = server.getPlayerManager().getPlayer(uuid);
        if (player != null) {
            return player.getName().getString();
        }

        // 尝试从用户缓存获取 - 现在正确处理Optional
        Optional<GameProfile> profile = server.getUserCache().getByUuid(uuid);
        if (profile.isPresent()) {
            return profile.get().getName();
        }

        return "Unknown";
    }

    private void loadData() {
        if (!Files.exists(dataFile)) {
            return;
        }

        try (Reader reader = Files.newBufferedReader(dataFile)) {
            JsonObject root = JsonParser.parseReader(reader).getAsJsonObject();
            for (Map.Entry<String, JsonElement> entry : root.entrySet()) {
                UUID uuid = UUID.fromString(entry.getKey());
                playerData.put(uuid, GSON.fromJson(entry.getValue(), PlayerTimeData.class));
            }
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法加载玩家在线时间数据", e);
        }
    }

    public void saveAll() {
        JsonObject root = new JsonObject();
        playerData.forEach((uuid, data) -> {
            root.add(uuid.toString(), GSON.toJsonTree(data));
        });

        try (Writer writer = Files.newBufferedWriter(dataFile)) {
            GSON.toJson(root, writer);
        } catch (Exception e) {
            PlayerTimeMod.LOGGER.error("[在线时间] 无法保存玩家在线时间数据", e);
        }
    }

    private void saveAsync(UUID uuid) {
        executor.execute(() -> {
            JsonObject root;
            try {
                if (Files.exists(dataFile)) {
                    try (Reader reader = Files.newBufferedReader(dataFile)) {
                        root = JsonParser.parseReader(reader).getAsJsonObject();
                    }
                } else {
                    root = new JsonObject();
                }
            } catch (Exception e) {
                root = new JsonObject();
            }

            PlayerTimeData data = playerData.get(uuid);
            if (data != null) {
                root.add(uuid.toString(), GSON.toJsonTree(data));
            }

            try (Writer writer = Files.newBufferedWriter(dataFile)) {
                GSON.toJson(root, writer);
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 无法保存" + uuid + "的在线时间数据", e);
            }
        });
    }

    public static String formatTime(long seconds) {
        long hours = seconds / 3600;
        long minutes = (seconds % 3600) / 60;
        return String.format("%dh %02dm", hours, minutes);
    }

    public static class PlayerTimeData {
        long totalTime = 0;
        long lastLogin = 0;
        RollingTimeWindow rolling30Days = new RollingTimeWindow(30);
        RollingTimeWindow rolling7Days = new RollingTimeWindow(7);
    }

    public static class PlayerTimeStats {
        public long totalTime;
        public long last30Days;
        public long last7Days;
    }

    private static class RollingTimeWindow {
        private final int days;
        private final List<TimeEntry> entries = new ArrayList<>();

        public RollingTimeWindow(int days) {
            this.days = days;
        }

        public void addPlayTime(long timestamp, long seconds) {
            entries.add(new TimeEntry(timestamp, seconds));
            cleanUp(timestamp);
        }

        public long getTotalTime(long currentTime) {
            cleanUp(currentTime);
            return entries.stream().mapToLong(e -> e.seconds).sum();
        }

        private void cleanUp(long currentTime) {
            long cutoff = currentTime - (days * 24 * 3600);
            entries.removeIf(entry -> entry.timestamp < cutoff);
        }

        private static class TimeEntry {
            final long timestamp;
            final long seconds;

            TimeEntry(long timestamp, long seconds) {
                this.timestamp = timestamp;
                this.seconds = seconds;
            }
        }
    }
    public Map<UUID, PlayerTimeData> getPlayerData() {
        return Collections.unmodifiableMap(playerData);
    }

}