package com.example.playertime;

import com.google.gson.*;
import com.sun.net.httpserver.HttpServer;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpExchange;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.PlayerManager;
import net.minecraft.server.network.ServerPlayerEntity;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.nio.file.*;
import java.util.*;
import java.util.concurrent.*;

import static com.mojang.text2speech.Narrator.LOGGER;

public class WebServer {
    private final HttpServer server;
    private final PlayerTimeTracker timeTracker;
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();
    private final ExecutorService executor = Executors.newFixedThreadPool(4);
    private final MinecraftServer minecraftServer;
    private static final Map<String, String> MIME_TYPES = Map.of(
            "html", "text/html",
            "css", "text/css",
            "js", "application/javascript",
            "json", "application/json"
    );

    public WebServer(PlayerTimeTracker timeTracker, int port, MinecraftServer minecraftServer) throws IOException {
        this.minecraftServer = minecraftServer;
        if (port < 1 || port > 65535) {
            throw new IllegalArgumentException("Invalid port number: " + port);
        }
        this.timeTracker = timeTracker;
        this.server = HttpServer.create(new InetSocketAddress(port), 0);
        setupContexts();
    }

    private void setupContexts() {
        server.createContext("/api/stats", exchange -> {
            exchange.getResponseHeaders().add("Access-Control-Allow-Origin", "*");
            exchange.getResponseHeaders().add("Access-Control-Allow-Methods", "GET, OPTIONS");
            exchange.getResponseHeaders().add("Access-Control-Allow-Headers", "Content-Type");


            if (!"GET".equals(exchange.getRequestMethod())) {
                sendResponse(exchange, 405, "Method Not Allowed");
                return;
            }

            try {
                // 改为使用新的白名单统计方法
                Map<String, String> stats = timeTracker.getWhitelistedPlayerStats();
                String response = new Gson().toJson(stats);
                sendResponse(exchange, 200, response.getBytes(), "application/json");
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 无法获得统计数据", e);
                sendResponse(exchange, 500, "Internal Server Error");
            }
        });

        // 静态文件服务
        server.createContext("/", exchange -> {
            try {
                String path = exchange.getRequestURI().getPath();
                if (path.equals("/")) path = "/index.html";

                // 从资源目录加载文件
                String resourcePath = "assets/playertime/web" + path;
                InputStream is = getClass().getClassLoader().getResourceAsStream(resourcePath);

                if (is == null) {
                    sendResponse(exchange, 404, "Not Found");
                    return;
                }

                // 确定内容类型
                String extension = path.substring(path.lastIndexOf('.') + 1);
                String contentType = MIME_TYPES.getOrDefault(extension, "text/plain");

                // 读取文件内容
                ByteArrayOutputStream buffer = new ByteArrayOutputStream();
                byte[] data = new byte[1024];
                int nRead;
                while ((nRead = is.read(data, 0, data.length)) != -1) {
                    buffer.write(data, 0, nRead);
                }
                buffer.flush();

                sendResponse(exchange, 200, buffer.toByteArray(), contentType);
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 无法提供资源", e);
                sendResponse(exchange, 500, "Internal Server Error");
            }
        });


        server.createContext("/api/widget-data", exchange -> {
            exchange.getResponseHeaders().add("Access-Control-Allow-Origin", "*");
            exchange.getResponseHeaders().add("Access-Control-Allow-Methods", "GET, OPTIONS");
            exchange.getResponseHeaders().add("Access-Control-Allow-Headers", "Content-Type");

            if ("OPTIONS".equals(exchange.getRequestMethod())) {
                exchange.sendResponseHeaders(204, -1); // 204 No Content
                return;
            }


            if (!"GET".equals(exchange.getRequestMethod())) {
                sendResponse(exchange, 405, "Method Not Allowed");
                return;
            }


            try {
                MinecraftServer server = minecraftServer;
                PlayerManager playerManager = server.getPlayerManager();

                JsonObject response = new JsonObject();
                response.addProperty("onlineCount", playerManager.getCurrentPlayerCount());

                // 获取白名单在线玩家
                JsonArray whitelistPlayers = new JsonArray();
                Map<String, Long> playerTimeMap = new HashMap<>();

                // 先收集所有白名单玩家UUID
                Set<UUID> whitelistUuids = new HashSet<>();
                for (String name : playerManager.getWhitelist().getNames()) {
                    server.getUserCache().findByName(name).ifPresent(profile -> {
                        whitelistUuids.add(profile.getId());
                    });
                }

                // 检查在线玩家
                for (ServerPlayerEntity player : playerManager.getPlayerList()) {
                    UUID uuid = player.getUuid();
                    if (whitelistUuids.contains(uuid)) {
                        PlayerTimeTracker.PlayerTimeStats stats = timeTracker.getPlayerStats(uuid);
                        if (stats != null) {
                            JsonObject playerJson = new JsonObject();
                            playerJson.addProperty("name", player.getName().getString());
                            playerJson.addProperty("time", PlayerTimeTracker.formatTime(stats.totalTime));
                            whitelistPlayers.add(playerJson);
                            playerTimeMap.put(player.getName().getString(), stats.totalTime);
                        }
                    }
                }
                response.add("whitelistPlayers", whitelistPlayers);

                // 获取时长前三玩家（包括离线玩家）
                JsonArray topPlayers = new JsonArray();
                timeTracker.getPlayerData().entrySet().stream()
                        .filter(entry -> whitelistUuids.contains(entry.getKey())) // 只筛选白名单玩家
                        .sorted((a, b) -> Long.compare(b.getValue().totalTime, a.getValue().totalTime))
                        .limit(3)
                        .forEach(entry -> {
                            JsonObject playerJson = new JsonObject();
                            playerJson.addProperty("name", timeTracker.getPlayerName(entry.getKey()));
                            playerJson.addProperty("time", PlayerTimeTracker.formatTime(entry.getValue().totalTime));
                            topPlayers.add(playerJson);
                        });

                response.add("topPlayers", topPlayers);
                response.addProperty("timestamp", System.currentTimeMillis());

                sendResponse(exchange, 200, GSON.toJson(response).getBytes(StandardCharsets.UTF_8), "application/json");
            } catch (Exception e) {
                PlayerTimeMod.LOGGER.error("[在线时间] 无法获得统计数据", e);
                sendResponse(exchange, 500, "Internal Server Error");
            }
        });





        server.setExecutor(executor);
    }

    private void sendResponse(HttpExchange exchange, int code, String response) throws IOException {
        sendResponse(exchange, code, response.getBytes(StandardCharsets.UTF_8), "text/plain");
    }

    private void sendResponse(HttpExchange exchange, int code, byte[] response, String contentType) throws IOException {
        exchange.getResponseHeaders().set("Access-Control-Allow-Origin", "*");
        exchange.getResponseHeaders().set("Access-Control-Allow-Methods", "GET, OPTIONS");
        exchange.getResponseHeaders().set("Access-Control-Allow-Headers", "Content-Type");


        exchange.getResponseHeaders().set("Content-Type", contentType);
        exchange.sendResponseHeaders(code, response.length);
        try (OutputStream os = exchange.getResponseBody()) {
            os.write(response);
        }
    }

    public void start() {
        server.start();
    }

    public void stop() {
        server.stop(0);
        executor.shutdown();
    }


    // 以前ai留下的烂方法，大概没用了，注释也是ai的

    private String generatePlayerList(Map<String, String> stats) {
        if (stats.isEmpty()) return "<p>暂无玩家数据</p>";

        StringBuilder sb = new StringBuilder("<ul style='list-style-type: none; padding-left: 5px;'>");
        stats.keySet().stream().limit(5).forEach(player -> {
            sb.append("<li>• ").append(player).append("</li>");
        });
        if (stats.size() > 5) {
            sb.append("<li>... 等 ").append(stats.size() - 5).append(" 位玩家</li>");
        }
        sb.append("</ul>");
        return sb.toString();
    }

    // 辅助方法：将"Xh Ym"格式的时间转换为秒数
    private long parseTimeToSeconds(String timeStr) {
        String[] parts = timeStr.split(" ");
        int hours = Integer.parseInt(parts[0].replace("h", ""));
        int minutes = parts.length > 1 ? Integer.parseInt(parts[1].replace("m", "")) : 0;
        return hours * 3600L + minutes * 60L;
    }

    // 读取时间数据文件
    private JsonObject readTimeData(Path dataFile) throws IOException {
        if (!Files.exists(dataFile)) {
            return new JsonObject();
        }

        try (Reader reader = Files.newBufferedReader(dataFile)) {
            return JsonParser.parseReader(reader).getAsJsonObject();
        }
    }

    // 获取玩家总时长
    private long getPlayerTotalTime(String uuid, JsonObject timeData) {
        if (!timeData.has(uuid)) {
            return 0;
        }

        JsonObject playerData = timeData.getAsJsonObject(uuid);
        if (playerData.has("totalTime")) {
            return playerData.get("totalTime").getAsLong();
        }
        return 0;
    }
}